package cc.fascinated.bat.features.logging.listeners;

import cc.fascinated.bat.common.EmbedDescriptionBuilder;
import cc.fascinated.bat.common.EmbedUtils;
import cc.fascinated.bat.common.EnumUtils;
import cc.fascinated.bat.event.EventListener;
import cc.fascinated.bat.features.logging.LogFeature;
import cc.fascinated.bat.features.logging.LogType;
import cc.fascinated.bat.model.BatGuild;
import cc.fascinated.bat.model.BatUser;
import lombok.NonNull;
import lombok.extern.log4j.Log4j2;
import net.dv8tion.jda.api.entities.channel.concrete.TextChannel;
import net.dv8tion.jda.api.entities.channel.concrete.VoiceChannel;
import net.dv8tion.jda.api.entities.channel.unions.AudioChannelUnion;
import net.dv8tion.jda.api.entities.channel.unions.ChannelUnion;
import net.dv8tion.jda.api.events.channel.ChannelCreateEvent;
import net.dv8tion.jda.api.events.channel.ChannelDeleteEvent;
import net.dv8tion.jda.api.events.guild.voice.GenericGuildVoiceEvent;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.Map;

/**
 * @author Fascinated (fascinated7)
 */
@Component
@Log4j2
public class ChannelListener implements EventListener {
    /**
     * A map of users and the last voice channel they were in
     */
    private final Map<BatUser, VoiceChannel> lastVoiceChannel = new HashMap<>();
    private final LogFeature logFeature;

    @Autowired
    public ChannelListener(@NonNull ApplicationContext context) {
        this.logFeature = context.getBean(LogFeature.class);
    }

    @Override
    public void onChannelCreate(@NonNull BatGuild guild, @NonNull ChannelCreateEvent event) {
        log.info("Channel \"{}\" was created in guild \"{}\"", event.getChannel().getName(), guild.getName());
        logFeature.sendLog(guild, LogType.CHANNEL_CREATE, EmbedUtils.successEmbed()
                .setDescription(new EmbedDescriptionBuilder("%s Channel Created".formatted(EnumUtils.getEnumName(event.getChannel().getType())))
                        .appendLine("Channel: %s".formatted(event.getChannel().getAsMention()), true)
                        .appendLine("Name: %s".formatted(event.getChannel().getName()), true)
                        .build())
                .build());
    }

    @Override
    public void onChannelDelete(@NonNull BatGuild guild, @NonNull ChannelDeleteEvent event) {
        log.info("Channel \"{}\" was deleted in guild \"{}\"", event.getChannel().getName(), guild.getName());
        ChannelUnion channel = event.getChannel();
        EmbedDescriptionBuilder description = new EmbedDescriptionBuilder("%s Channel Deleted".formatted(EnumUtils.getEnumName(channel.getType())))
                .appendLine("Name: #%s".formatted(channel.getName()), true);
        if (channel instanceof TextChannel) {
            TextChannel textChannel = channel.asTextChannel();
            description.appendLine("Topic: %s".formatted(textChannel.getTopic()), true);
        }
        logFeature.sendLog(guild, LogType.CHANNEL_DELETE, EmbedUtils.errorEmbed().setDescription(description.build()).build());
    }

    @Override
    public void onGuildVoiceUpdate(@NonNull BatGuild guild, @NonNull BatUser user, @NonNull GenericGuildVoiceEvent event) {
        AudioChannelUnion channel = event.getVoiceState().getChannel();
        if (channel != null) {
            VoiceChannel voiceChannel = channel.asVoiceChannel();
            lastVoiceChannel.put(user, voiceChannel);
        }
        VoiceChannel voiceChannel = lastVoiceChannel.get(user);
        if (voiceChannel == null) {
            return;
        }
        boolean joined = voiceChannel.getMembers().contains(event.getMember());
        if (!joined) {
            lastVoiceChannel.remove(user);
        }
        log.info("User \"{}\" {} voice channel \"{}\" in guild \"{}\"", user.getName(), joined ? "joined" : "left", voiceChannel.getName(), guild.getName());
        String description = new EmbedDescriptionBuilder("User %s Voice Channel".formatted(joined ? "Joined" : "Left"))
                .appendLine("User: %s".formatted(user.getDiscordUser().getAsMention()), true)
                .appendLine("Channel: %s".formatted(voiceChannel.getAsMention()), true)
                .build();
        if (joined) {
            logFeature.sendLog(guild, LogType.VOICE_CHANNEL_JOIN, EmbedUtils.successEmbed().setDescription(description).build());
            return;
        }
        logFeature.sendLog(guild, LogType.VOICE_CHANNEL_LEAVE, EmbedUtils.errorEmbed().setDescription(description).build());
    }
}
