package cc.fascinated.bat.features.spotify.command;

import cc.fascinated.bat.Emojis;
import cc.fascinated.bat.command.BatSubCommand;
import cc.fascinated.bat.command.CommandInfo;
import cc.fascinated.bat.event.EventListener;
import cc.fascinated.bat.exception.BatException;
import cc.fascinated.bat.features.spotify.SpotifyFeature;
import cc.fascinated.bat.model.BatGuild;
import cc.fascinated.bat.model.BatUser;
import cc.fascinated.bat.service.SpotifyService;
import lombok.NonNull;
import lombok.SneakyThrows;
import lombok.extern.log4j.Log4j2;
import net.dv8tion.jda.api.entities.Member;
import net.dv8tion.jda.api.entities.channel.middleman.MessageChannel;
import net.dv8tion.jda.api.events.interaction.component.ButtonInteractionEvent;
import net.dv8tion.jda.api.interactions.commands.SlashCommandInteraction;
import net.dv8tion.jda.api.interactions.components.ActionRow;
import net.dv8tion.jda.api.interactions.components.LayoutComponent;
import net.dv8tion.jda.api.interactions.components.buttons.Button;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.concurrent.TimeUnit;

/**
 * @author Fascinated (fascinated7)
 */
@Component
@Log4j2
@CommandInfo(name = "current", description = "Gets the currently playing Spotify track")
public class CurrentSubCommand extends BatSubCommand implements EventListener {
    private final SpotifyService spotifyService;

    @Autowired
    public CurrentSubCommand(@NonNull SpotifyService spotifyService) {
        this.spotifyService = spotifyService;
    }

    @Override
    public void execute(BatGuild guild, @NonNull BatUser user, @NonNull MessageChannel channel, Member member, @NonNull SlashCommandInteraction event) {
        event.replyEmbeds(SpotifyFeature.currentSong(spotifyService, user).build()).addComponents(createActions()).queue(message -> {
            message.editOriginalComponents(new ArrayList<>()).queueAfter(5, TimeUnit.MINUTES); // Remove the buttons after 5 minutes
        });
    }

    @Override @SneakyThrows
    public void onButtonInteraction(BatGuild guild, @NonNull BatUser user, @NonNull ButtonInteractionEvent event) {
        if (!event.getUser().getId().equals(user.getId())) {
            throw new BatException("%s You cannot interact with this message.".formatted(Emojis.CROSS_MARK_EMOJI));
        }

        if (!event.getComponentId().equals("pause")
                && !event.getComponentId().equals("resume")
                && !event.getComponentId().equals("skip")) {
            return;
        }

        switch (event.getComponentId()) {
            case "pause" -> {
                event.replyEmbeds(SpotifyFeature.pauseSong(spotifyService, user)
                                .appendDescription("\nThis message will disappear <t:%s:R>".formatted(System.currentTimeMillis() / 1000 + 15)).build())
                        .queue(message -> {
                            message.deleteOriginal().queueAfter(15, TimeUnit.SECONDS);
                        });
            }
            case "resume" -> {
                event.replyEmbeds(SpotifyFeature.resumeSong(spotifyService, user)
                                .appendDescription("\nThis message will disappear <t:%s:R>".formatted(System.currentTimeMillis() / 1000 + 15)).build())
                        .queue(message -> {
                            message.deleteOriginal().queueAfter(15, TimeUnit.SECONDS);
                        });
            }
            case "skip" -> {
                event.replyEmbeds(SpotifyFeature.skipSong(spotifyService, user)
                                .appendDescription("\nThis message will disappear <t:%s:R>".formatted(System.currentTimeMillis() / 1000 + 15)).build())
                        .queue(then -> {
                            then.deleteOriginal().queueAfter(15, TimeUnit.SECONDS);
                            event.getInteraction().getMessage().editMessageEmbeds(SpotifyFeature.currentSong(spotifyService, user).build()).queue();
                        });
            }
        }
    }

    /**
     * Creates the action buttons for the currently playing track
     *
     * @return The action buttons
     */
    private LayoutComponent[] createActions() {
        return new LayoutComponent[]{
                ActionRow.of(
                        Button.primary("pause", "Pause").withEmoji(Emojis.PAUSE_EMOJI),
                        Button.primary("resume", "Resume").withEmoji(Emojis.PLAY_EMOJI),
                        Button.primary("skip", "Skip").withEmoji(Emojis.SKIP_EMOJI)
                )
        };
    }
}
