package cc.fascinated.bat.features.tmdb.command;

import cc.fascinated.bat.command.BatSubCommand;
import cc.fascinated.bat.command.CommandInfo;
import cc.fascinated.bat.common.EmbedUtils;
import cc.fascinated.bat.event.EventListener;
import cc.fascinated.bat.features.tmdb.TMDBFeature;
import cc.fascinated.bat.model.BatGuild;
import cc.fascinated.bat.model.BatUser;
import cc.fascinated.bat.service.TMDBService;
import lombok.NonNull;
import net.dv8tion.jda.api.EmbedBuilder;
import net.dv8tion.jda.api.entities.Member;
import net.dv8tion.jda.api.entities.channel.concrete.TextChannel;
import net.dv8tion.jda.api.entities.channel.middleman.MessageChannel;
import net.dv8tion.jda.api.entities.emoji.Emoji;
import net.dv8tion.jda.api.events.interaction.component.ButtonInteractionEvent;
import net.dv8tion.jda.api.interactions.commands.OptionMapping;
import net.dv8tion.jda.api.interactions.commands.OptionType;
import net.dv8tion.jda.api.interactions.commands.SlashCommandInteraction;
import net.dv8tion.jda.api.interactions.components.buttons.Button;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.Map;

@Component
@CommandInfo(name = "series", description = "Get information about a series")
public class SeriesSubCommand extends BatSubCommand implements EventListener {

    private final TMDBService tmdbService;
    private final Map<String, Map<String, String>> userCommands; // Map to store user commands and their parameters

    @Autowired
    public SeriesSubCommand(@NonNull TMDBService tmdbService) {
        this.tmdbService = tmdbService;
        this.userCommands = new HashMap<>();

        super.addOption(OptionType.STRING, "title", "The title of the series", true);
        super.addOption(OptionType.STRING, "language", "A locale code (en-US) to lookup movies in a specific language", false);
        super.addOption(OptionType.INTEGER, "first_air_year", "Filter the results so that only the first air year has this value", false);
        super.addOption(OptionType.INTEGER, "year", "Filter the results release dates to matches that include this value", false);
    }

    @Override
    public void execute(BatGuild guild, @NonNull BatUser user, @NonNull MessageChannel channel, Member member, @NonNull SlashCommandInteraction event) {
        OptionMapping titleOption = event.getOption("title");
        if (titleOption == null) {
            event.replyEmbeds(EmbedUtils.errorEmbed()
                            .setDescription("You must provide a title to search for!")
                            .build())
                    .queue();
            return;
        }

        // Determine if the channel is NSFW. If so, allow adult content
        boolean adult = false;
        if (event.getChannel() instanceof TextChannel textChannel) {
            adult = textChannel.isNSFW();
        }

        OptionMapping languageOption = event.getOption("language");
        OptionMapping firstAirYearOption = event.getOption("first_air_year");
        OptionMapping yearOption = event.getOption("year");

        // Store user command and parameters for later use
        Map<String, String> params = new HashMap<>();
        params.put("title", titleOption.getAsString());
        if (languageOption != null) params.put("language", languageOption.getAsString());
        if (firstAirYearOption != null) params.put("first_air_year", String.valueOf(firstAirYearOption.getAsInt()));
        if (yearOption != null) params.put("year", String.valueOf(yearOption.getAsInt()));
        params.put("adult", String.valueOf(adult));

        EmbedBuilder embedBuilder = TMDBFeature.pageSeries(
                tmdbService,
                titleOption.getAsString(),
                (languageOption != null ? languageOption.getAsString() : null),
                (firstAirYearOption != null ? firstAirYearOption.getAsInt() : -1),
                (yearOption != null ? yearOption.getAsInt() : -1),
                0, // Initial page number
                adult
        );

        if (embedBuilder == null) {
            event.replyEmbeds(EmbedUtils.errorEmbed()
                            .setDescription("No series found with the given parameters!")
                            .build())
                    .queue();
            return;
        }

        userCommands.put(user.getId(), params);
        event.replyEmbeds(embedBuilder.build()
        ).addActionRow(
                Button.primary("backSeries", "Back").withEmoji(Emoji.fromFormatted("⬅️")),
                Button.primary("nextSeries", "Next").withEmoji(Emoji.fromFormatted("➡️"))
        ).queue();
    }

    @Override
    public void onButtonInteraction(BatGuild guild, @NonNull BatUser user, @NonNull ButtonInteractionEvent event) {
        Map<String, String> params = userCommands.get(user.getId());
        if (params == null) {
            return;
        }

        int currentPage = Integer.parseInt(params.getOrDefault("page", "0"));
        boolean adult = Boolean.parseBoolean(params.get("adult"));

        // Retrieve stored parameters
        String title = params.get("title");
        String language = params.get("language");
        int firstAirYear = (params.get("first_air_year") != null ? Integer.parseInt(params.get("first_air_year")) : -1);
        int year = (params.get("year") != null ? Integer.parseInt(params.get("year")) : -1);

        if (event.getComponentId().equals("backSeries")) {
            currentPage--;
            if (currentPage < 0) {
                currentPage = 0; // Ensure currentPage doesn't go negative
            }
        } else if (event.getComponentId().equals("nextSeries")) {
            currentPage++;
        }

        params.put("page", String.valueOf(currentPage));

        EmbedBuilder embedBuilder = TMDBFeature.pageSeries(
                tmdbService,
                title,
                language,
                firstAirYear,
                year,
                currentPage,
                adult
        );
        if (embedBuilder == null) {
            event.editMessageEmbeds(EmbedUtils.errorEmbed()
                            .setDescription("No series found with the given parameters!")
                            .build())
                    .queue();
            return;
        }
        event.editMessageEmbeds(embedBuilder.build()).queue();
    }
}